#include <iostream>
#include <functional>
#include <cmath>
#include <array>
#include <complex>

#include "matrix.h"

using namespace std;

template<class T>
Matrix<T>& Matrix<T>::operator=(Matrix<T> const& cpy)
{
  if(&cpy == this)
    {
      return *this;
    }
  
  data = cpy.data;
  siz = cpy.siz;
  return *this;
}

template<class T>
Matrix<T>&& Matrix<T>::operator=(Matrix<T>&& mv)
{
  if(&mv == this)
    {
      return *this;
    }

  siz = mv.siz;
  data = std::move(mv.data);
  return *this;
}
  
template<class T>
Matrix<T> operator+(Matrix<T> const& left, Matrix<T> const& right)
{
  return Matrix<T>([&](unsigned int x){ return left[x] + right[x]; },
		   left.size());
}

template<class T>
Matrix<T>&& operator+(Matrix<T>&& left, Matrix<T> const& right)
{
  std::transform(left.data.begin(),
		 left.data.end(),
		 right.data.begin(),
		 left.data.begin(),
		 [](T const& x, T const& y){ return x + y; });
  return std::move(left);
}

template<class T>
Matrix<T>& Matrix<T>::operator+=(Matrix<T> const& other)
{
  std::transform(this->data.begin(),
		 this->data.end(),
		 other.data.begin(),
		 this->data.begin(),
		 [](T const& x, T const& y){ return x + y; });
  return *this;
}

template<class T>
Matrix<T> operator-(Matrix<T> const& left, Matrix<T> const& right)
{
  return Matrix<T>([&](unsigned int x){ return left[x] - right[x]; },
		   left.size());
}

template<class T>
Matrix<T>&& operator-(Matrix<T>&& left, Matrix<T> const& right)
{
  std::transform(left.data.begin(),
		 left.data.end(),
		 right.data.begin(),
		 left.data.begin(),
		 [](T const& x, T const& y){ return x - y; });
  return std::move(left);
}

template<class T>
Matrix<T>& Matrix<T>::operator-=(Matrix<T> const& other)
{
  std::transform(this->data.begin(),
		 this->data.end(),
		 other.data.begin(),
		 this->data.begin(),
		 [](T const& x, T const& y){ return x - y; });

  return *this;
}

template<class T>
Matrix<T> operator/(Matrix<T> const& A, T const& s)
{
  return Matrix<T>([&](unsigned int i)
                   {
		     return A[i] / s;
		   },
                   A.size());
}

template<class T>
Matrix<T>&& operator/(Matrix<T>&& A, T const& s)
{
  std::for_each(A.data.begin(),
		A.data.end(),
		[&](T & x){ return x /= s; });
  return std::move(A);
}


template<class T>
Matrix<T>& Matrix<T>::operator/=(T const& s)
{
  std::for_each(this->data.begin(),
		this->data.end(),
		[&](T & x){ return x /= s; });
  return *this;
}

template<class T>
Matrix<T> operator*(Matrix<T> const& A, T const& s)
{
  return Matrix<T>([&](unsigned int i)
                   {
		     return A[i] * s;
		   },
                   A.size());
}

template<class T>
Matrix<T>&& operator*(Matrix<T>&& A, T const& s)
{
  std::for_each(A.data.begin(),
		A.data.end(),
		[&](T & x){ return x *= s; });
  return std::move(A);
}

template<class T>
Matrix<T>& Matrix<T>::operator*=(T const& s)
{
  std::for_each(this->data.begin(),
		this->data.end(),
		[&](T & x){ return x *= s; });
  return *this;
}

template<class T>
Matrix<T> operator*(Matrix<T> const& left, Matrix<T> const& right)
{
  auto N = left.size();
  return
    Matrix<T>([&](int i, int j)
              {
		T sum = static_cast<T>(0.0);
		for(unsigned int k = 0; k < N; k++)
		  {
		    sum += left(i, k) * right(k, j);
		  }
		return sum;
	      },
              N);
}
template<class T>
std::ostream& operator<<(std::ostream& out, Matrix<T> const& arg)
{
  auto N = arg.size();
  for(unsigned int i = 0; i < N; i++)
    {
      for(unsigned int j = 0; j < N; j++)
	{
	  out << arg(j, i) << "\t";
	}
      out << std::endl;
    }
  return out;
}

int main(int, char**)
{
  auto M = Matrix<int>([](int i)
                       {
			 return i+1;
		       },
                       2);
  auto N = Matrix<int>([](int i)
                       {
			 return i-1;
		       },
                       2);
  
  cout << (M * 2) - (M / 2) * 2 << std::endl;
  cout << (M * M) << std::endl;
  cout << (M *= 2) << std::endl;
  cout << (M -= N) << std::endl;
  auto oof = Matrix<int>([](int i, int j)
                         {
			   return (i-1)*(j-1);
			 },
                         2) + N;
  cout << N << std::endl;
  cout << oof << std::endl;
  
  return 0;
}
