#include <iostream>
#include <functional>
#include <tuple>
#include <cmath>

using namespace std;

template<class T, unsigned int N=2u>
T adaptive_newton_cotes(function<T(T)> integrand,
			function<T(function<T(T)>, T, T)> approximation,
			T lower,
			T upper,
			T tolerance)
{
    static_assert(N > 1, "N must at least 2");
    
    T sum = static_cast<T>(0.0);
    T h = (upper - lower) / N;
    
    T interval_lower, interval_upper, interval_middle;
    T interval_integral, lower_half_integral, upper_half_integral;

    // divide the whole interval into N subintervals
    for(unsigned int i = 0; i < N; i++)
    {
	// divide the subinterval into 2 subdivisions
	interval_lower = lower + i * h;
	interval_middle = lower + (i+0.5) * h;
	interval_upper = lower + (i+1) * h;
	
	    
	interval_integral   = approximation(integrand, interval_lower, interval_upper);
	lower_half_integral = approximation(integrand, interval_lower, interval_middle);
	upper_half_integral = approximation(integrand, interval_middle, interval_upper);
	
	cout << "Margin of error on interval [" << interval_lower << "\t.. " << interval_upper <<  "\t]: " <<
	    fabs(interval_integral-(lower_half_integral+upper_half_integral)) << endl;
	
	// compare the subinterval to the 2 subdivisions
	if(fabs(interval_integral-(lower_half_integral+upper_half_integral)) <= tolerance)
	{
	    sum += interval_integral;
	}
	else
	{
	    cout << "Dividing the interval..." << endl;
	    sum += adaptive_newton_cotes<T>(integrand,
					    approximation,
					    interval_lower,
					    interval_upper,
					    tolerance);
	}
    }
    // dunno why but it calculates double...
    return sum;
}


template<class T>
T simpson_approximation(function<T(T)> f, T lower, T upper)
{
    return 1.0/6.0 * (upper - lower) * (f(lower) + 4*f((upper+lower)/2) + f(upper));
}

template<class T>
T trapezoidal_approximation(function<T(T)> f, T lower, T upper)
{
    return (upper - lower) * (f(lower) + f(upper)) / 2.0;
}

int main(int, char**)
{
    auto poly = [](double z)->double {return -z*z*z - z*z + 4*z - 6;};
    //auto test = [](double z)->double {return z*z*(z + 1);};
    auto integral = adaptive_newton_cotes<double, 20u>(poly, trapezoidal_approximation<double>, 0.1, 1.0, 0.1);
    cout << "The integral on [0.1, 1.0] is: " << integral << endl;
    return 0;
}
