#include <iostream>
#include <functional>
#include <cmath>
#include <limits>

const double epsilon = 0.01;

using namespace std;

template <class T, int max_iterations = 1000>
T newton_iteration(function<T(T)> f, function<T(T)> derivative, function<bool(function<T(T)>, T, T)> stop_predicate, T x0)
{
  T x=x0;
  // technically we don't need this
  T prev_x = numeric_limits<T>::infinity();
  int i = 1;
  
  // I wanted the loop to run at least once
  do
  {    
    if(i > max_iterations) throw domain_error("Iteration didn't reach a stable point.");
    
    if(derivative(x) == 0) throw domain_error("Function is constant at the point.");
    
    prev_x = x;
    // if f is linear, then dx*f(x)/dx = f(x) and f(x) instantly becomes zero
    x-=f(x)/derivative(x);;
    
    cout << "Iteration " << i << ":\t" << "x = " << x << "\tdfx = " << derivative(x) << endl;

    i++;
  } while(stop_predicate(f,x, prev_x));
  return x;
}

template <class T>
bool metric_predicate(function<T(T)> f, T prev, T curr)
{
  return (fabs(f(curr)-f(prev)) > epsilon);
}

double sqrt_newton(double k)
{
  auto sqrt_finder = [&k](double z)->double {return z*z - k;};
  auto derivative  = [](double z)->double {return 2*z;};
  return newton_iteration<double, 1000>(sqrt_finder, derivative, metric_predicate<double>, +40.0); 
}


int main(void)
{
  // this has root at z=-3 and no other real roots
  auto poly  = [](double z)->double {return -z*z*z - z*z + 4*z - 6;};
  auto dpoly = [](double z)->double {return -3*z*z - 2*z + 4;};
  
  double x0 = newton_iteration<double, 1000>(poly, dpoly, metric_predicate<double>, +40.0);
  cout << "Function nears zero around x = " << x0 << endl;
  double k = 321;
  double root = sqrt_newton(k);
  cout << "The square root of " << k << " is " << root << endl;
  return 0;
}
