#include <iostream>
#include <functional>
#include <cmath>
#include <array>
#include <complex>
#include "vector.h"
using namespace std;

template<class T>
Vector2D<T>& Vector2D<T>::operator+=(const Vector2D<T>& other)
{
  coords[0] += other.coords[0];
  coords[1] += other.coords[1];
  return *this;
};

template<class T>
Vector2D<T>& Vector2D<T>::operator-=(const Vector2D<T>& other)
{
  coords[0] -= other.coords[0];
  coords[1] -= other.coords[1];
  return *this;
};

// prolly only for commutative product, i don't think it's needed because can't be chained
template<class T>
Vector2D<T>& Vector2D<T>::operator*=(const T& scalar)
{
  coords[0] *= scalar;
  coords[1] *= scalar;
  return *this;
};

template<class T>
Vector2D<T> operator+(const Vector2D<T>& left, const Vector2D<T>& right)
{
  return Vector2D<T>(left.coords[0] + right.coords[0], left.coords[1] + right.coords[1]);
};

template<class T>
Vector2D<T> operator-(const Vector2D<T>& left, const Vector2D<T>& right)
{
  return Vector2D<T>(left.coords[0] - right.coords[0], left.coords[1] - right.coords[1]);
};

template<class T>
Vector2D<T> operator*(const T& scalar, const Vector2D<T>& vector)
{
  return Vector2D<T>(scalar * vector.coords[0], scalar * vector.coords[1]);
};

template<class T>
ostream& operator<<(ostream& os, const Vector2D<T>& vector)
{
  return os << "(" << vector.coords[0] << ", " << vector.coords[1] << ")" ;
};

template<class T>
istream& operator>>(istream& is, Vector2D<T>& vector)
{
  is >> vector.coords[0];
  is >> vector.coords[1];
  return is;
};

template<class T>
T euclidean_norm(const Vector2D<T>& v)
{
  return dot_product(v,v);
}

template<class T>
T dot_product(const Vector2D<T>& vl, const Vector2D<T>& vr)
{
  return
    (vl.coords[0] * conj(vr.coords[0])) +
    (vl.coords[1] * conj(vr.coords[1]));
}

template<class T>
Vector2D<T> normalized(const Vector2D<T>& v, function<T(const Vector2D<T> &)> norm_function)
{
  T norm = norm_function(v);
  return (static_cast<T>(1)/norm) * v;
}


int main(int, char**)
{
  using namespace std::complex_literals;
  typedef complex<double> C;
  Vector2D<C> first(1. + 2i, 3. + 1i);
  Vector2D<C> second(-1. - 2i, 3. + 1i);
  C fugg(-1.0 - 0.5i);

  cout << euclidean_norm<C>(fugg * first + second) << endl;
  cout << dot_product(first, second) << endl;
 
  cout << normalized(first+second, euclidean_norm<C>) << endl;
  return 0;
}
