#include <cmath>
#include <iostream>
#include <functional>

#include <popt.h>

using namespace std;

double omega;          // the natural frequency
double x, v;           // position and velocity at time t
int periods;           // number of periods to integrate
int steps;    // number of time steps dt per period

void getInput(void);                             // for user to input parameters
void EulerCromer(double dt);                 // takes an Euler-Cromer step
void Euler(double dt);                       // takes an Euler step
double energy(void);                             // computes the energy
void sim(function<void(double)> integrator); // runs the simulation

typedef double v2d __attribute__ ((vector_size (2*8)));

enum {
	OK=0,
	USAGE=1,
	HOSTFAIL=2,
};

void EulerCromer (double dt) {
  double a = - omega * omega * x;
	v += a * dt;
	x += v * dt;
}

void Euler(double dt) {
	double a = - omega * omega * x;
  x += v * dt;
  v += a * dt;
}

double energy(void) {
    return 0.5 * (v * v + omega * omega * x * x);
}

void sim(function<void(double)> integrator) {
	const double pi = 4 * atan(1.0);
  double T = 2 * pi / omega;
  double dt = T / steps;
  double t = 0;
  cout << t << '\t' << x << '\t' << v << '\t' << energy() << '\n';
  for (int p = 1; p <= periods; p++) {
    for (int s = 0; s < steps; s++) {
			integrator(dt);
			t += dt;
			cout << t << '\t' << x << '\t' << v << '\t' << energy() << '\n';
		}
    // cout << "Period = " << p << "\tt = " << t
    // cout << "\tx = " << x << "\tv = " << v
    // cout << "\tenergy = " << energy() << endl;
  }
}

int main (int argc, const char *argv[]) {
	const struct poptOption options_table[] = {
    {"omega",   'o', POPT_ARG_DOUBLE,   &omega, 0,    "Value of omega", NULL},
    {"xnull",   'x', POPT_ARG_DOUBLE,       &x, 0,     "Value of x(0)", NULL},
    {"vnull",   'v', POPT_ARG_DOUBLE,       &v, 0,     "Value of v(0)", NULL},
    {"periods", 'p', POPT_ARG_INT,    &periods, 0, "Number of periods", NULL},
    {"steps",   's', POPT_ARG_INT,      &steps, 0,  "Steps per period", NULL},
    POPT_AUTOHELP
    {NULL, 0, 0, NULL, 0, NULL, NULL}};

  poptContext opt_con = poptGetContext("sho", argc, argv, options_table, 0);
  poptSetOtherOptionHelp(opt_con, "[OPTIONS]*");
	poptGetNextOpt(opt_con);
  sim(Euler);
	return EXIT_SUCCESS;
}
